// Ansteuerung eines HD44780 kompatiblen LCD im 4-Bit-Interfacemodus
// http://www.mikrocontroller.net/articles/AVR-GCC-Tutorial
//
// Die Pinbelegung ist ber defines in lcd-routines.h einstellbar
 

#include "main.h"






//------------- sendet einen Integerwert an das LCD-1 ---------------------------
void lcd1_putint(int value){ 
 char tmp[20]; 
	
        itoa(value, tmp, 10);
   		lcd1_string(tmp);
		lcd1_string(" ");
		lcd1_string(" ");
		lcd1_string(" ");
}


//-------------zahl im lcd ausgeben----LCD-1---------------------
void lcd1_putword(char theword[])
{
        int i = 0;
        int j = 0;

        while (theword[i] != '\0') // char nicht null
		{
                j = i;

                if (theword[i] == '\n') //char ist null
					{
                        while (j < 40)
								{
                                lcd1_data(' '); // lcd space einfgen
                                j++;
                        		}
                     i++;
                	}
                lcd1_data(theword[i]); //zahl an lcd ausgeben
                i++;
        }
}



 
//----------------- sendet ein Datenbyte an das LCD-1---------------------
 
void lcd1_data(unsigned char temp1)
{
   unsigned char temp2 = temp1;
    LCD_DDR = LCD_DDR | 0x0F | (1<<LCD_RS) ;   // DataPort auf Ausgang schalten

 
   LCD_DATA_PORT |= (1<<LCD_RS);        // RS auf 1 setzen
 
   temp1 = temp1 >> 4;
   temp1 = temp1 & 0x0F;
   LCD_DATA_PORT &= 0xF0;
   LCD_DATA_PORT |= temp1;               // setzen
   lcd1_enable();
 
   temp2 = temp2 & 0x0F;
   LCD_DATA_PORT &= 0xF0;
   LCD_DATA_PORT |= temp2;               // setzen
   lcd1_enable();
   
   _delay_us(42);
}
 
//--------------- sendet einen Befehl an das LCD-1 -------------------------------
 
void lcd1_command(unsigned char temp1)
{
   unsigned char temp2 = temp1;
    LCD_DDR = LCD_DDR | 0x0F | (1<<LCD_RS) ;   // Port als Ausgang schalten

 
   LCD_DATA_PORT &= ~(1<<LCD_RS);        // RS auf 0 setzen
 
   temp1 = temp1 >> 4;              // oberes Nibble holen
   temp1 = temp1 & 0x0F;            // maskieren
   LCD_DATA_PORT &= 0xF0;
   LCD_DATA_PORT |= temp1;               // setzen
   lcd1_enable();
 
   temp2 = temp2 & 0x0F;            // unteres Nibble holen und maskieren
   LCD_DATA_PORT &= 0xF0;
   LCD_DATA_PORT |= temp2;               // setzen
   lcd1_enable();
   
   _delay_us(42);
}
 

 
//----------------- Initialisierung --LCD-1------------------------------- 
// Muss ganz am Anfang des Programms aufgerufen werden.
 
void lcd1_init(void)
{
   LCD_DDR = LCD_DDR | 0x0F | (1<<LCD_RS) ;   // Port als Ausgang schalten

 
   // muss 3mal hintereinander gesendet werden zur Initialisierung
 
   _delay_ms(15);
   LCD_DATA_PORT &= 0xF0;
   LCD_DATA_PORT |= 0x03;            
   LCD_DATA_PORT &= ~(1<<LCD_RS);      // RS auf 0
   lcd1_enable();
 
   _delay_ms(5);
   lcd1_enable();
 
   _delay_ms(1);
   lcd1_enable();
   _delay_ms(1);
 
   // 4 Bit Modus aktivieren 
   LCD_DATA_PORT &= 0xF0;
   LCD_DATA_PORT |= 0x02;
   lcd1_enable();
   _delay_ms(1);
 
   // 4Bit / 2 Zeilen / 5x7
   lcd1_command(0x28);
    
   // Display ein / Cursor aus / kein Blinken
   lcd1_command(0x0C); 
 
   // inkrement / kein Scrollen
   lcd1_command(0x06);
 
   lcd1_clear();
}
 
//------------ Sendet den Befehl zur Lschung des Displays -LCD-1----------------------
 
void lcd1_clear(void)
{
   lcd1_command(CLEAR_DISPLAY);
   _delay_ms(5);
}
 
//----------- Sendet den Befehl: Cursor Home ---LCD-1---------------------------------
 
void lcd1_home(void)
{
   lcd1_command(CURSOR_HOME);
   _delay_ms(5);
}
 
//------------- setzt den Cursor in Zeile y (1..4) Spalte x (0..15) LCD-1-------------------
 
void set_cursor1(uint8_t x, uint8_t y)
{
  uint8_t tmp;
 
  switch (y) {
    case 1: tmp=0x80+0x00+x; break;    // 1. Zeile
    case 2: tmp=0x80+0x40+x; break;    // 2. Zeile
    case 3: tmp=0x80+0x14+x; break;    // 3. Zeile
    case 4: tmp=0x80+0x54+x; break;    // 4. Zeile
    default: return;                   // fr den Fall einer falschen Zeile
  }
  lcd1_command(tmp);
}
 
//---------------- Schreibt einen String auf das LCD-1 ----------------------------------
 
void lcd1_string(char *data)
{
    while(*data) {
        lcd1_data(*data);
        data++;
    }
}

//--------------- sendet einen Befehl an das LCD-2 -------------------------------
 
void lcd2_command(unsigned char temp1)
{
   unsigned char temp2 = temp1;
    LCD_DDR = LCD_DDR | 0x0F | (1<<LCD_RS) ;   // Port als Ausgang schalten

 
   LCD_DATA_PORT &= ~(1<<LCD_RS);        // RS auf 0 setzen
 
   temp1 = temp1 >> 4;              // oberes Nibble holen
   temp1 = temp1 & 0x0F;            // maskieren
   LCD_DATA_PORT &= 0xF0;
   LCD_DATA_PORT |= temp1;               // setzen
   lcd2_enable();
 
   temp2 = temp2 & 0x0F;            // unteres Nibble holen und maskieren
   LCD_DATA_PORT &= 0xF0;
   LCD_DATA_PORT |= temp2;               // setzen
   lcd2_enable();
   
   _delay_us(42);
}

//----------------- Initialisierung --LCD-2------------------------------- 
// Muss ganz am Anfang des Programms aufgerufen werden.
 
void lcd2_init(void)
{
   LCD_DDR = LCD_DDR | 0x0F | (1<<LCD_RS) ;   // Port als Ausgang schalten
  
   // muss 3mal hintereinander gesendet werden zur Initialisierung
 
   _delay_ms(15);
   LCD_DATA_PORT &= 0xF0;
   LCD_DATA_PORT |= 0x03;            
   LCD_DATA_PORT &= ~(1<<LCD_RS);      // RS auf 0
   lcd2_enable();
 
   _delay_ms(5);
   lcd2_enable();
 
   _delay_ms(1);
   lcd2_enable();
   _delay_ms(1);
 
   // 4 Bit Modus aktivieren 
   LCD_DATA_PORT &= 0xF0;
   LCD_DATA_PORT |= 0x02;
   lcd2_enable();
   _delay_ms(1);
 
   // 4Bit / 2 Zeilen / 5x7
   lcd2_command(0x28);
    
   // Display ein / Cursor aus / kein Blinken
   lcd2_command(0x0C); 
 
   // inkrement / kein Scrollen
   lcd2_command(0x06);
 
   lcd2_clear();
}

//------------- setzt den Cursor in Zeile y (1..4) Spalte x (0..15) LCD-2-------------------
 
void set_cursor2(uint8_t x, uint8_t y)
{
  uint8_t tmp;
 
  switch (y) {
    case 1: tmp=0x80+0x00+x; break;    // 1. Zeile
    case 2: tmp=0x80+0x40+x; break;    // 2. Zeile
    case 3: tmp=0x80+0x14+x; break;    // 3. Zeile
    case 4: tmp=0x80+0x54+x; break;    // 4. Zeile
    default: return;                   // fr den Fall einer falschen Zeile
  }
  lcd2_command(tmp);
}

//------------- sendet einen Integerwert an das LCD-2 ---------------------------
void lcd2_putint(int value){ 
 char tmp[20]; 
	
        itoa(value, tmp, 10);
   		lcd2_string(tmp);
		lcd2_string(" ");
		lcd2_string(" ");
		lcd2_string(" ");
}

//-------------zahl im lcd ausgeben----LCD-2---------------------
void lcd2_putword(char theword[])
{
        int i = 0;
        int j = 0;

        while (theword[i] != '\0') // char nicht null
		{
                j = i;

                if (theword[i] == '\n') //char ist null
					{
                        while (j < 40)
								{
                                lcd1_data(' '); // lcd space einfgen

                                j++;
                        		}
                     i++;

                	}
                lcd2_data(theword[i]); //zahl an lcd ausgeben
                i++;
        }
}

//----------------- sendet ein Datenbyte an das LCD-2---------------------
 
void lcd2_data(unsigned char temp1)
{
   unsigned char temp2 = temp1;
    LCD_DDR = LCD_DDR | 0x0F | (1<<LCD_RS) ;   // Port als Ausgang schalten
 
   LCD_DATA_PORT |= (1<<LCD_RS);        // RS auf 1 setzen
 
   temp1 = temp1 >> 4;
   temp1 = temp1 & 0x0F;
   LCD_DATA_PORT &= 0xF0;
   LCD_DATA_PORT |= temp1;               // setzen
   lcd2_enable();
 
   temp2 = temp2 & 0x0F;
   LCD_DATA_PORT &= 0xF0;
   LCD_DATA_PORT |= temp2;               // setzen
   lcd2_enable();
   
   _delay_us(42);
}

//----------- Sendet den Befehl: Cursor Home -LCD-2------------------------
void lcd2_home(void)
{
   lcd2_command(CURSOR_HOME);
   _delay_ms(5);
}

//------------ Sendet den Befehl zur Lschung des Displays -LCD-2----------
void lcd2_clear(void)
{
   lcd2_command(CLEAR_DISPLAY);
   _delay_ms(5);
}

//------------- erzeugt den Enable-Puls LCD-1  -----------------------------
void lcd1_enable(void)
{
//	LCD_STEUER_DDR |=(1<<LCD_EN1);	// Port als Ausgang schalten
//	asm("nop");

	LCD_STEUER_PORT |= (1<<LCD_EN1);
    _delay_us(4);      				// kurze Pause 
   	LCD_STEUER_PORT &= ~(1<<LCD_EN1);
                
	
} 

//------------- erzeugt den Enable-Puls LCD-2  ---------------------------------
void lcd2_enable(void)
{
//	LCD_STEUER_DDR |=(1<<LCD_EN2);	// Port als Ausgang schalten
//	asm("nop");
   
  	LCD_STEUER_PORT |=  (1<<LCD_EN2);
    _delay_us(4);   	                // kurze Pause    
	LCD_STEUER_PORT &= ~(1<<LCD_EN2);
}
 
//---------------- Schreibt einen String auf das LCD-2 ----------------------------------
 
void lcd2_string(char *data)
{
    while(*data) {
        lcd2_data(*data);
        data++;
    }
}
